import { IDocument, TAsyncDeleteDocument, TAsyncGetDocument, TAsyncPostDocument, TAsyncPutDocument, Uid } from '@react_db_client/constants.client-types';
import { ICallback, AsyncRequestError } from '@react_db_client/async-hooks.use-async-request';
export interface IDeleteResponse {
    ok?: boolean;
}
export interface ISaveResponse {
    ok?: boolean;
}
export interface IUseAsyncObjectManagerArgs<DocType extends IDocument> {
    activeUid?: null | Uid;
    collection: string;
    isNew?: boolean;
    inputAdditionalData?: null | Partial<DocType>;
    schema?: string | 'all';
    populate?: 'all' | string[];
    loadOnInit?: boolean;
    reloadOnSave?: boolean;
    onSavedCallback?: (uid: Uid, response: any, combinedData: DocType) => void;
    saveErrorCallback?: (e: AsyncRequestError) => void;
    onDeleteCallback?: ICallback<IDeleteResponse, [string, Uid]>;
    asyncGetDocument: TAsyncGetDocument<DocType>;
    asyncPutDocument: TAsyncPutDocument<DocType>;
    asyncPostDocument: TAsyncPostDocument<DocType>;
    asyncDeleteDocument: TAsyncDeleteDocument;
}
export interface IUseAsyncObjectManagerReturn<DocType extends IDocument> {
    loadedData: null | Partial<DocType>;
    saveData: () => void;
    updateData: (newData: Partial<DocType>) => void;
    updateFormData: (field: Uid, value: any, save?: boolean, nested?: boolean) => void;
    resetData: () => void;
    reload: () => void;
    deleteObject: () => void;
    saveResponse: null | ISaveResponse;
    deleteResponse?: null | IDeleteResponse;
    loadingData: boolean;
    savingData: boolean;
    deletingData: boolean;
    data: Partial<DocType>;
    initialData: Partial<DocType>;
    uid: Uid;
    callCount: number;
    hasLoaded: boolean;
    unsavedChanges: boolean;
    isNew: boolean;
}
/**
 * Async React request hook
 *
 * @param {Object} {
 *   - `activeUid` {string} - the active object uid
 *   - `collection` {string} - the collection to load and save to
 *   - `isNew` {bool} - true if the object is new
 *   - `inputAdditionalData` {object} - additional data to be merged with the loaded data
 *   - `schema` {string|Array} - schema override for loading data
 *   - `loadOnInit` {bool} - true if we should load the object on component init
 * }
 * @return {Object} {
 *   - `saveData` {func} - save object function
 *   - `updateData` {func} - update edit data ()
 *   - `updateFormData` = (field, value, save = false) => {...}
 *   - `reload` {func} - reload and reset the object
 *   - `deleteObject` {func} - delete the object
 *   - `saveResponse` {string} - api response to saving the object
 *   - `deleteResponse` {func} - api response to deleting the object
 *   - `loadingData` {bool} - true when loading data
 *   - `savingData` {bool} - true when saving data
 *   - `deletingData` {bool} - true when deleting data
 *   - `data` {any} - the data returned from api and combined with additional data
 *   - `uid` {string} - the active uid
 *   - `callCount` {number} - number of timesi called
 *   - `hasLoaded` {bool} - true if we have loaded data
 * }
 */
export declare const useAsyncObjectManager: {
    <DocType extends IDocument>({ activeUid, collection, isNew: isNewIn, inputAdditionalData, schema, populate, loadOnInit, reloadOnSave, onSavedCallback: onSavedCallbackIn, saveErrorCallback, onDeleteCallback, asyncGetDocument, asyncPutDocument, asyncPostDocument, asyncDeleteDocument, }: IUseAsyncObjectManagerArgs<DocType>): IUseAsyncObjectManagerReturn<DocType>;
    propTypes: {
        activeUid: any;
        collection: any;
        isNew: any;
        inputAdditionalData: any;
        schema: any;
        loadOnInit: any;
        reloadOnSave: any;
        onSavedCallback: any;
        saveErrorCallback: any;
        asyncGetDocument: any;
        asyncPutDocument: any;
        asyncPostDocument: any;
        asyncDeleteDocument: any;
    };
};
