"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable camelcase */
require("@samnbuk/react_db_client.testing.enzyme-setup");
const react_1 = __importDefault(require("react"));
const react_hooks_1 = require("@testing-library/react-hooks");
const enzyme_1 = require("enzyme");
const use_async_object_manager_composition_1 = require("./use-async-object-manager.composition");
const use_async_object_manager_1 = require("./use-async-object-manager");
const test_utils_1 = require("react-dom/test-utils");
Date.now = jest.fn().mockImplementation(() => 0);
const loadedData = { loaded: 'data' };
const asyncGetDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return loadedData; }));
const asyncPutDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return ({ ok: true }); }));
const asyncPostDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return ({ ok: true }); }));
const asyncDeleteDocument = jest.fn().mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return ({ ok: true }); }));
const onSavedCallback = jest.fn();
const defaultArgs = {
    activeUid: 'demouid',
    collection: 'democollection',
    isNew: false,
    inputAdditionalData: {},
    schema: 'all',
    populate: 'all',
    loadOnInit: true,
    asyncGetDocument,
    asyncPutDocument,
    asyncPostDocument,
    asyncDeleteDocument,
    onSavedCallback,
};
describe.skip('useAsyncObjectManager', () => {
    beforeAll(() => {
        jest.useFakeTimers();
    });
    afterAll(() => {
        jest.useRealTimers();
    });
    describe('Compositions', () => {
        test('should render asyncTest1', () => {
            (0, test_utils_1.act)(() => {
                (0, enzyme_1.mount)(react_1.default.createElement(use_async_object_manager_composition_1.AsyncTest1, null));
            });
            (0, test_utils_1.act)(() => {
                jest.runOnlyPendingTimers();
            });
        });
        /* TODO: Fix this test as having async issues */
        test.skip('should render asyncTest2', () => {
            (0, test_utils_1.act)(() => {
                (0, enzyme_1.mount)(react_1.default.createElement(use_async_object_manager_composition_1.AsyncTest2, null));
            });
            (0, test_utils_1.act)(() => {
                jest.runOnlyPendingTimers();
            });
        });
    });
    describe('Functional Tests', () => {
        beforeEach(() => {
            asyncGetDocument.mockClear();
            asyncPutDocument.mockClear();
            asyncPostDocument.mockClear();
            asyncDeleteDocument.mockClear();
            onSavedCallback.mockClear();
        });
        test('should call async get docs', () => __awaiter(void 0, void 0, void 0, function* () {
            const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
            expect(asyncGetDocument).toHaveBeenCalledWith(defaultArgs.collection, defaultArgs.activeUid, defaultArgs.schema, defaultArgs.populate);
            expect(result.current.loadedData).toEqual(null);
            expect(result.current.data).toEqual(Object.assign(Object.assign({}, defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
            yield waitForNextUpdate();
            expect(result.current.loadedData).toEqual(loadedData);
            expect(result.current.data).toEqual(Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
        }));
        test('should save data', () => __awaiter(void 0, void 0, void 0, function* () {
            const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
            yield waitForNextUpdate();
            (0, react_hooks_1.act)(() => {
                result.current.saveData();
            });
            yield waitForNextUpdate();
            (0, react_hooks_1.act)(() => {
                jest.runOnlyPendingTimers();
            });
            expect(asyncPutDocument).toHaveBeenCalledWith(defaultArgs.collection, defaultArgs.activeUid, Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
        }));
        test('should update form data', () => __awaiter(void 0, void 0, void 0, function* () {
            const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
            yield waitForNextUpdate();
            expect(result.current.data).toEqual(Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
            const editField = 'edit';
            const editValue = 'val';
            (0, react_hooks_1.act)(() => {
                result.current.updateFormData(editField, editValue);
            });
            expect(result.current.data).toEqual(Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid, [editField]: editValue }));
        }));
        test('should update form data and save', () => __awaiter(void 0, void 0, void 0, function* () {
            const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(defaultArgs));
            yield waitForNextUpdate();
            expect(onSavedCallback).not.toHaveBeenCalled();
            expect(result.current.data).toEqual(Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
            asyncGetDocument.mockClear();
            /* Update field */
            const editField = 'edit';
            const editValue = 'val';
            (0, react_hooks_1.act)(() => {
                result.current.updateFormData(editField, editValue, true);
            });
            yield waitForNextUpdate();
            /* put document is called */
            const combinedData = Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid, [editField]: editValue });
            expect(asyncPutDocument).toHaveBeenCalledWith(defaultArgs.collection, defaultArgs.activeUid, combinedData);
            /* On save callback called */
            expect(onSavedCallback).toHaveBeenCalledWith(defaultArgs.activeUid, { ok: true }, combinedData);
            expect(onSavedCallback).toHaveBeenCalledTimes(1);
            expect(asyncGetDocument).not.toHaveBeenCalled();
            /* Final data should be the reloaded data with all other changes cleared */
            expect(result.current.data).toEqual(combinedData);
        }));
        test('should update form data and save then reload', () => __awaiter(void 0, void 0, void 0, function* () {
            const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, use_async_object_manager_1.useAsyncObjectManager)(Object.assign(Object.assign({}, defaultArgs), { reloadOnSave: true })));
            yield waitForNextUpdate();
            expect(result.current.data).toEqual(Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
            /* We provide alternate load data to show that data is reloaded */
            const altLoadedData = Object.assign(Object.assign({}, loadedData), { altdata: 'altdata' });
            asyncGetDocument.mockClear();
            asyncGetDocument.mockImplementation(() => __awaiter(void 0, void 0, void 0, function* () { return altLoadedData; }));
            /* Update field */
            const editField = 'edit';
            const editValue = 'val';
            (0, react_hooks_1.act)(() => {
                result.current.updateFormData(editField, editValue, true);
            });
            yield waitForNextUpdate();
            // /* put document is called */
            const combinedData = Object.assign(Object.assign(Object.assign({}, loadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid, [editField]: editValue });
            expect(asyncPutDocument).toHaveBeenCalledWith(defaultArgs.collection, defaultArgs.activeUid, combinedData);
            /* On save callback called */
            expect(onSavedCallback).toHaveBeenCalledWith(defaultArgs.activeUid, { ok: true }, combinedData);
            expect(onSavedCallback).toHaveBeenCalledTimes(1);
            /* We now reload data */
            expect(asyncGetDocument).toHaveBeenCalledWith(defaultArgs.collection, defaultArgs.activeUid, 'all', 'all');
            /* Final data should be the reloaded data with all other changes claered */
            expect(result.current.data).toEqual(Object.assign(Object.assign(Object.assign({}, altLoadedData), defaultArgs.inputAdditionalData), { uid: defaultArgs.activeUid }));
        }));
    });
});
//# sourceMappingURL=use-async-object-manager.fn.test.js.map